// Main window
"use strict";
// globals: document, GT, requestAnimationFrame, window, DH, setInterval, FuriganaBrowser, setTimeout

var GT = GT || {};

// duplicate tiles for single tile characters
GT.characters.singleTileCharacter('waterfront');
GT.characters.singleTileCharacter('invisible');
GT.characters.singleTileCharacter('crystals');

GT.customLoop = function () {
    // Custom rendering loop
    if (GT.background.map) {
        GT.render();
    }
    // loop itself
    requestAnimationFrame(GT.customLoop);
};

GT.onHerb = function () {
    // Use medicinal herb to heal itself
    if (GT.characters.player.herb > 0 && GT.characters.player.health < 100) {
        GT.characters.player.herb--;
        GT.characters.player.health = 100;
        document.getElementById('herbbutton').innerText = GT.characters.player.herb;
        GT.metrics.mission.herbsUsed++;
    }
};

GT.onAttack = function (event) {
    // Attack nearest enemy
    // animate punch
    GT.effects.attack(GT.characters.player);
    // find nearest enemy
    var near = GT.characters.nearest(GT.characters.player, {"slime": 1, "frog": 1, "crystals": 1, "ghost": 1, "skeleton": 1, "bat": 1, "spider": 1, "rat": 1}, 1.5, true);
    if (near) {
        // hit enemy (take 40 health, except in ghost mission take as much to kill ghost in 30 hits)
        near.health -= GT.mission.major >= 11 ? GT.characters.defaults.ghost.health / 30 : 40;
        // animate hit
        GT.effects.hit(near);
        // did it died?
        if (near.health <= 0) {
            near.turn('dead');
            GT.mission.sort();
            // undead fix
            setTimeout(function () {
                near.updateTile(GT.time);
            }, 500);
            setTimeout(function () {
                near.updateTile(GT.time);
            }, 1500);
            GT.metrics.mission.kills++;
        }
        GT.metrics.mission.attacks++;
    } else {
        GT.metrics.mission.misses++;
    }
    // prevent double attack with mouse/touch
    event.cancelBubble = true;
    event.preventDefault();
};

GT.die = function () {
    // Player death
    GT.characters.player.bubble = "Aaargh..";
    GT.characters.player.turn('dead');
    // disable controls
    GT.keyboard.touchpad = false;
    GT.touchpad.x = 0;
    GT.touchpad.y = 0;
    GT.touchpad.hide();
    // game over
    setTimeout(function () {
        GT.characters.player.bubble = "";
        GT.mission.gameOver();
    }, 2000);
};

function go(aPlace) {
    // Go to a place (used many times during development)
    // first look on current map
    var m, p, xy;
    try {
        p = GT.mission.placeOrNpc(GT.characters.player.map, aPlace);
        if (p) {
            GT.characters.player.teleport(p.map, p.x, p.y);
            return;
        }
        // then on all maps
        for (m in GT.maps) {
            if (GT.maps.hasOwnProperty(m)) {
                p = GT.mission.placeOrNpc(m, aPlace);
                if (p) {
                    GT.characters.player.teleport(p.map, p.x, p.y);
                    return;
                }
            }
        }
    } catch (e) {
        console.error(e);
        // then just first place on map
        if (GT.maps.hasOwnProperty(aPlace)) {
            xy = Object.keys(GT.maps[aPlace].place)[0].split(' ');
            GT.characters.player.teleport(aPlace, parseInt(xy[0], 10), parseInt(xy[1], 10));
            return;
        }
    }
}

function test() {
    // Start test mission
    GT.mission.major = 20;
    GT.mission.save();
    document.location.reload();
}

function purge() {
    // Erase storage and reload app
    DH.storage.eraseAll();
    document.location.reload();
}

function fwd() {
    // Show fwd button
    DH.storage.writeBoolean('GT.fwd', true);
    document.location.reload();
}

// initialize window
window.addEventListener('DOMContentLoaded', function () {
    try {
        // buttons
        document.getElementById('minimap').addEventListener('click', GT.mission.minimap);
        document.getElementById('attack').addEventListener('touchstart', GT.onAttack);
        document.getElementById('attack').addEventListener('mousedown', GT.onAttack);
        document.getElementById('herb').addEventListener('touchstart', GT.onHerb);
        document.getElementById('herb').addEventListener('mousedown', GT.onHerb);
        document.getElementById('objective').addEventListener('click', GT.menu);

        GT.init(function () {
            // initialize canvas
            GT.canvas.init('background_canvas', 'character_canvas');

            // initialize on-screen touchpad
            GT.touchpad = DH.touchpad('icon/arrows130.png', undefined, true);
            GT.touchpad.img.style.zIndex = 10;

            // rendering loop
            GT.customLoop();

            // start last mission
            GT.mission.run();
        });

        // update healthbar
        var health = document.getElementById('health');
        setInterval(function () {
            if (GT.characters.player) {
                health.value = GT.characters.player.health;
            }
        }, 200);

        // show fwd button only when enabled
        document.getElementById('fwd').style.visibility = DH.storage.readBoolean('GT.fwd', false) ? 'visible' : 'hidden';
        document.getElementById('con').style.visibility = DH.storage.readBoolean('GT.con', false) ? 'visible' : 'hidden';
        document.getElementById('con').addEventListener('click', function () {
            DH.console.show(true);
        });
    } catch (e) {
        alert(e);
    }
});

