// Dummy Android object so that non-android apps won't crash trying calling it
// require: none, import
"use strict";
// globals: Android

if (typeof window.Android !== 'object') {
    console.log('Creating dummy Android object');

    window.Android = (function () {
        var self = {},
            back = 'file:///android_asset/android.html',
            build = '2017-11-30 10:05:00 build 1',
            data = '',
            history = [];

        self.hasPermissionWriteExternalStorage = function () {
            // desktop always has permission
            return true;
        };

        self.askPermissionWriteExternalStorage = function () {
            // desktop always has permission
            self.permissionCallback(true);
        };

        self.isReal = function () {
            // return false because this is not real android
            console.log('Android.isReal', false);
            return false;
        };

        self.loadUrl = function (aUrl) {
            // change webview url
            console.log('Android.loadUrl', aUrl);
            document.location = aUrl.toString().replace('file:///android_asset/', '');
        };

        self.getBuild = function () {
            // return unique build identifier (usually date)
            console.log('Android.getBuild', build);
            return build;
        };

        self.setUserAgent = function (aUserAgent) {
            // set location where to go when user press back button
            console.log('Android.setUserAgent', aUserAgent);
        };

        self.setBack = function (aUrl) {
            // set location where to go when user press back button
            console.log('Android.setBack', aUrl);
            back = aUrl;
        };

        self.getBack = function () {
            // return current url set on back button
            console.log('Android.getBack', back);
            return back;
        };

        self.setShared = function (aData) {
            // set shared data
            console.log('Android.setShared', aData);
            localStorage.setItem('Android.setShared', aData);
        };

        self.getShared = function () {
            // get shared data
            var s = localStorage.getItem('Android.shared');
            console.log('Android.getShared', s);
            return s;
        };

        self.setData = function (aData) {
            // set callback data
            data = aData;
            console.log('Android.setData', aData.length);
            localStorage.setItem('Android.setData', aData);
        };

        self.getData = function () {
            // get callback data
            console.log('Android.getData', data.length);
            return data;
        };

        self.showToast = function (aMessage) {
            // Show small piece of text at the bottom of screen
            console.log('Android.showToast', aMessage);
            var div, toast;
            div = document.createElement('div');
            div.style.position = 'fixed';
            div.style.bottom = '1cm';
            div.style.left = '1cm';
            div.style.right = '1cm';
            div.style.bottom = '1cm';
            div.style.zIndex = 111;
            div.style.textAlign = 'center';
            toast = document.createElement('div');
            toast.textContent = aMessage;
            toast.style.display = 'inline-block';
            toast.style.backgroundColor = 'rgba(0,0,0,0.7)';
            toast.style.color = 'white';
            toast.style.padding = '1ex';
            toast.style.borderRadius = '2ex';
            document.body.appendChild(div);
            setTimeout(function () {
                div.parentElement.removeChild(div);
            }, 5000);
            div.appendChild(toast);
        };

        self.keyboardShow = function () {
            // show soft keyboard
            console.log('Android.keyboardShow');
        };

        self.keyboardHide = function () {
            // hide soft keyboard
            console.log('Android.keyboardHide');
        };

        self.keyboardImplicit = function () {
            // show soft keyboard (version 2)
            console.log('Android.keyboardImplicit');
        };

        self.dataCallback = function () {
            // called when chooseFile is ready
            console.log('Android.dataCallback');
        };

        self.chooseFile = function (aFilter) {
            // let user choose file
            console.log('Android.chooseFile', aFilter);
            DH.import(function (aData) {
                data = aData;
                self.dataCallback();
            }, function () {
                data = '';
                self.dataCallback();
            });
        };

        self.getHistory = function () {
            return history.join('\n');
        };

        self.requests = function () {
            return ["http://localhost/", "http://example.com/", "http://example.com/favicon.ico"].join('\n');
        };

        self.historyPop = function () {
            return history.pop();
        };

        return self;
    }());
}

Android.withPermissionWriteExternalStorage = function (aCallback, aCallbackData) {
    // run callback if user has write permission
    var b = Android.hasPermissionWriteExternalStorage();
    console.log('b', b, typeof b, typeof Android.hasPermissionWriteExternalStorage);
    if (b) {
        console.log('has permission on start');
        aCallback(aCallbackData);
        return;
    }
    Android.permissionCallback = function (aGranted) {
        console.log('granted', aGranted);
        if (aGranted) {
            aCallback(aCallbackData);
        } else {
            Android.showToast('No write permission!');
        }
    };
    Android.askPermissionWriteExternalStorage();
};
