// Fake console for WebView, this will allow user to see console on mobile using console.show()
// require: none
"use strict";
// globals: document, localStorage, window, Android

var DH = DH || {};

DH.consoleOld = console;
DH.console = (function () {
    var self = {
        limit: 200,
        buffer: [],
        important: []
    }, clicks = 0, clicktime = 0, showRun = false;

    self.stringify = function (aObject, aMaxDepth, aFoo, aIndent) {
        // stringify object e.g. max 2 level deep
        var t, k, fl;
        aFoo = aFoo || [];
        aIndent = aIndent || '';
        for (k in aObject) {
            t = typeof aObject[k];
            switch (t) {
            case "string":
            case "number":
            case "boolean":
                aFoo.push(aIndent + k + ': ' + aObject[k]);
                break;
            default:
                if (aObject[k] === null) {
                    aFoo.push(aIndent + k + ': null');
                } else {
                    if (Array.isArray(aObject[k])) {
                        aFoo.push(aIndent + k + ': [' + aObject[k].toString() + ']');
                    } else {
                        if (aMaxDepth > 1) {
                            aFoo.push(aIndent + k + ': {');
                            fl = aFoo.length;
                            self.stringify(aObject[k], aMaxDepth - 1, aFoo, aIndent + '    ');
                            if (aFoo.length === fl) {
                                aFoo[aFoo.length - 1] += '}';
                            } else {
                                aFoo.push(aIndent + '}');
                            }

                        } else {
                            aFoo.push(aIndent + k + ': {' + t + '}');
                        }
                    }
                }
            }
        }
        return aFoo.join('\n');
    };

    function push(aMessage) {
        if (aMessage.error || aMessage.warn) {
            self.important.push(aMessage);
        }
        self.buffer.push(aMessage);
        try {
            if ((self.limit > 0) && (self.buffer.length > self.limit)) {
                var a = self.buffer.length;
                self.buffer.splice(0, self.buffer.length - self.limit);
                if (a === self.buffer.length) {
                    alert('this is not possible');
                }
            }
        } catch (e) {
            alert(e);
        }
    }

    self.log = function () {
        push({ log: arguments });
        DH.consoleOld.log(arguments);
        return arguments;
    };

    self.warn = function () {
        push({ warn: arguments });
        DH.consoleOld.warn(arguments);
        return arguments;
    };

    self.info = function () {
        push({ info: arguments });
        DH.consoleOld.info(arguments);
        return arguments;
    };

    self.error = function () {
        push({ error: arguments });
        DH.consoleOld.error(arguments);
        return arguments;
    };

    self.debug = function () {
        push({ debug: arguments });
        DH.consoleOld.debug(arguments);
        return arguments;
    };

    self.clear = function () {
        self.buffer = [];
        DH.consoleOld.clear(arguments);
    };

    self.showRaw = function (aData) {
        // show any raw data on screen
        var t;
        t = document.createElement('div');
        t.style.backgroundColor = 'white';
        t.style.display = 'block';
        t.style.position = 'fixed';
        t.style.left = '0';
        t.style.right = '0';
        t.style.top = '0';
        t.style.bottom = '0';
        t.style.zIndex = 107;
        t.style.whiteSpace = 'pre';
        t.style.overflowY = 'scroll';
        t.style.fontFamily = 'monospace';
        t.style.color = 'black';
        t.style.backgroundColor = 'white';
        t.textContent = aData;
        t.ondblclick = function () {
            t.parentElement.removeChild(t);
        };
        document.body.appendChild(t);
        return t;
    };

    self.show = function (aShowRun) {
        var div = document.createElement('div'), i, k, line, s, j, a, clear, close, run, cd, output, color = {
            "log": "black",
            "info": "blue",
            "warn": "orange",
            "error": "red",
            "debug": "green",
            "extra": "lime"
        };

        function onClose() {
            div.parentElement.removeChild(div);
        }

        div.style.position = 'fixed';
        div.style.left = 0;
        div.style.top = 0;
        div.style.right = 0;
        div.style.bottom = 0;
        div.style.zIndex = 107;
        div.style.color = 'black';
        div.style.backgroundColor = 'white';
        div.style.boxSizing = 'border-box';
        div.style.overflowY = 'scroll';

        clear = document.createElement('button');
        clear.textContent = 'Clear';
        clear.style.minHeight = '1cm';
        clear.addEventListener('click', function () {
            self.important = [];
            self.buffer = [];
            onClose();
        });
        div.appendChild(clear);

        close = document.createElement('button');
        close.textContent = 'Close';
        close.style.minHeight = '1cm';
        close.style.float = 'right';
        close.addEventListener('click', onClose);
        div.appendChild(close);

        run = document.createElement('button');
        run.textContent = 'Run';
        run.style.minHeight = '1cm';
        run.addEventListener('click', function () {
            var cmd = prompt('Code', '');
            if (cmd) {
                try {
                    output.textContent = eval(cmd);
                    output.style.border = '1px solid blue';
                } catch (e) {
                    output.textContent = e;
                    output.style.border = '1px solid red';
                }
            }
        });
        if (aShowRun) {
            div.appendChild(run);
        }

        if (typeof Android === 'object') {
            cd = document.createElement('button');
            cd.textContent = 'CD';
            cd.style.minHeight = '1cm';
            cd.addEventListener('click', function () {
                output.textContent = Android.consoleData();
            });
            div.appendChild(cd);
        }

        output = document.createElement('div');
        div.appendChild(output);

        // important first
        for (i = 0; i < self.important.length; i++) {
            s = [];
            for (k in self.important[i]) {
                if (self.important[i].hasOwnProperty(k)) {
                    a = self.important[i][k];
                    for (j in a) {
                        if (a.hasOwnProperty(j)) {
                            s.push(a[j] && a[j].toString());
                        }
                    }
                    line = document.createElement('div');
                    line.textContent = s.join(' ');
                    line.style.color = color.hasOwnProperty(k) ? color[k] : "purple";
                    div.appendChild(line);
                }
            }
        }
        // all messages
        for (i = 0; i < self.buffer.length; i++) {
            s = [];
            for (k in self.buffer[i]) {
                if (self.buffer[i].hasOwnProperty(k)) {
                    s.push(i + ': ');
                    a = self.buffer[i][k];
                    for (j in a) {
                        if (a.hasOwnProperty(j)) {
                            s.push(a[j] && a[j].toString());
                        }
                    }
                    line = document.createElement('div');
                    line.textContent = s.join(' ');
                    line.style.color = color.hasOwnProperty(k) ? color[k] : "purple";
                    div.appendChild(line);
                }
            }
        }
        document.body.appendChild(div);
        return div;
    };

    self.dir = function () {
        self.log('console.dir not implemented');
    };

    self.dirxml = function () {
        self.log('console.dirxml not implemented');
    };

    self.table = function () {
        self.log('console.table not implemented');
    };

    self.trace = function () {
        self.log('console.trace not implemented');
    };

    self.group = function () {
        self.log('console.group not implemented');
    };

    self.groupCollapsed = function () {
        self.log('console.groupCollapsed not implemented');
    };

    self.groupEnd = function () {
        self.log('console.groupEnd not implemented');
    };

    self.count = function () {
        self.log('console.count not implemented');
    };

    self.assert = function () {
        self.log('console.assert' + 't not implemented'); // linter
    };

    self.markTimeline = function () {
        self.log('console.markTimeline not implemented');
    };

    self.profile = function () {
        self.log('console.profile not implemented');
    };

    self.profileEnd = function () {
        self.log('console.profileEnd not implemented');
    };

    self.timeline = function () {
        self.log('console.timeline not implemented');
    };

    self.timelineEnd = function () {
        self.log('console.timelineEnd not implemented');
    };

    self.time = function () {
        self.log('console.time not implemented');
    };

    self.timeEnd = function () {
        self.log('console.timeEnd not implemented');
    };

    self.timeStamp = function () {
        self.log('console.timeStamp not implemented');
    };

    self.memory = DH.consoleOld.memory;

    self.enable = function () {
        // enable using DH.console as regular console
        localStorage.setItem('DH.console', '1');
    };

    self.disable = function () {
        // disable using DH.console as regular console
        localStorage.setItem('DH.console', '0');
    };

    function onWindowClick(event) {
        // detect 10 fast clicks
        if (event.timeStamp > clicktime + 300) {
            clicks = 0;
        }
        clicks++;
        if (clicks > 10) {
            self.show(showRun);
            clicks = 0;
        }
        clicktime = event.timeStamp;
    }

    self.showOnMultipleClicks = function (aShowRun, aElement) {
        // make console show after fast 10 consecutive clicks
        showRun = aShowRun;
        (aElement || window).addEventListener('click', onWindowClick);
    };

    return self;
}());

if (!localStorage.hasOwnProperty('DH.console') || (localStorage.getItem('DH.console') === '1')) {
    console = DH.console;
}
