// Show dialog with key-value options
// require: none
"use strict";

var DH = DH || {};

DH.options = function (aOptions, aHints, aCallback, aTitle) {
    // show options dialog
    var div, h1, cancel, k, form, table, tr, td, inp,
        opt, o, opts, inputs = {}, focus = null;

    div = document.createElement('div');
    div.className = 'dh_options';
    div.style.position = 'fixed';
    div.style.left = 0;
    div.style.top = 0;
    div.style.right = 0;
    div.style.bottom = 0;
    div.style.display = 'flex';
    div.style.flexDirection = 'column';
    div.style.backgroundColor = 'white';
    div.style.color = 'black';
    div.style.padding = '1ex';
    div.style.margin = '0';

    h1 = document.createElement('h1');
    h1.textContent = aTitle || 'Options';
    h1.style.margin = '0';
    h1.style.padding = '0';
    h1.style.paddingBottom = '1ex';
    h1.style.fontSize = 'large';
    div.appendChild(h1);

    // for some reason table cannot have flex=1
    form = document.createElement('form');
    form.style.flex = '1';
    form.style.overflowY = 'scroll';
    div.appendChild(form);

    table = document.createElement('table');
    table.style.marginRight = '1ex';
    form.appendChild(table);

    function hint(aKey, aHint, aDefault) {
        // get single hint
        if (aHints && aHints.hasOwnProperty(aKey) && aHints[aKey].hasOwnProperty(aHint)) {
            return aHints[aKey][aHint];
        }
        return aDefault;
    }

    for (k in aOptions) {
        if (aOptions.hasOwnProperty(k)) {
            // row
            tr = document.createElement('tr');
            table.appendChild(tr);
            // first column
            td = document.createElement('td');
            if (aHints && aHints.hasOwnProperty(k) && aHints[k].caption) {
                td.textContent = aHints[k].caption;
            } else {
                td.textContent = k;
            }
            td.style.whiteSpace = 'nowrap';
            tr.appendChild(td);
            // second column
            td = document.createElement('td');
            td.style.width = '100%';
            tr.appendChild(td);
            // input
            opts = hint(k, 'options');
            if (opts) {
                inp = document.createElement('select');
                for (o = 0; o < opts.length; o++) {
                    opt = document.createElement('option');
                    opt.textContent = opts[o];
                    inp.appendChild(opt);
                }
            } else {
                inp = document.createElement('input');
            }
            inputs[k] = inp;
            if (inp.nodeName.toUpperCase() === 'INPUT') {
                inp.type = hint(k, 'type', 'text');
            }
            if (hint(k, 'maxLength') > 0) {
                inp.maxLength = hint(k, 'maxLength');
            }
            if (['text', 'url', 'email', 'password'].indexOf(inp.type) >= 0) {
                if (hint(k, 'maxLength', 0) <= 0) {
                    inp.style.width = '100%';
                }
            }
            // min max
            if (hint(k, 'min')) {
                inp.min = hint(k, 'min');
            }
            if (hint(k, 'max')) {
                inp.max = hint(k, 'max');
            }
            // required
            if (hint(k, 'required')) {
                inp.required = true;
            }
            // focus
            if (hint(k, 'focus')) {
                focus = inp;
            }
            // datalist options
            /*
            opts = hint(k, 'options', []);
            if (opts.length > 0) {
                dl = document.createElement('datalist');
                for (o = 0; o < opts.length; o++) {
                    opt = document.createElement('option');
                    opt.textContent = opts[o];
                    dl.appendChild(opt);
                }
                dl.id = 'datalist-for-' + k;
                div.appendChild(dl);
                inp.setAttribute('list', 'datalist-for-' + k);
                inp.style.width = '';
            }
            */
            inp.value = aOptions[k];
            td.appendChild(inp);

            // note
            if (hint(k, 'note')) {
                tr = document.createElement('tr');
                table.appendChild(tr);
                // first column
                td = document.createElement('td');
                tr.appendChild(td);
                // second column
                td = document.createElement('td');
                td.style.width = '100%';
                td.textContent = hint(k, 'note');
                tr.appendChild(td);
            }
        }
    }

    // submit button
    tr = document.createElement('tr');
    table.appendChild(tr);
    // first column
    td = document.createElement('td');
    tr.appendChild(td);
    // second column
    td = document.createElement('td');
    td.style.width = '100%';
    tr.appendChild(td);
    inp = document.createElement('input');
    inp.type = 'submit';
    inp.value = 'Save';
    td.appendChild(inp);

    // cancel button
    cancel = document.createElement('button');
    cancel.style.marginLeft = '1ex';
    cancel.textContent = 'Cancel';
    td.appendChild(cancel);
    cancel.onclick = function (event) {
        event.preventDefault();
        div.parentElement.removeChild(div);
        aCallback();
    };

    // submit function (this will also run input validation)
    form.onsubmit = function (event) {
        event.preventDefault();
        var j;
        for (j in aOptions) {
            if (aOptions.hasOwnProperty(j)) {
                aOptions[j] = inputs[j].value;
            }
        }
        if (aCallback) {
            aCallback(aOptions);
        }
        div.parentElement.removeChild(div);
    };

    document.body.appendChild(div);
    setTimeout(function () { if (focus) { focus.focus(); } }, 300);
};

