// Maps journal (changes, persistence) and overlapping
"use strict";
// globals: document, window, DH

var GT = GT || {};

GT.map = (function () {
    // Maps journal (changes, persistence) and overlapping
    var self = {};
    self.journal = DH.storage.readObject('GT.map.journal', {});

    self.save = function () {
        // Save journal to local storage
        DH.storage.writeObject('GT.map.journal', self.journal);
    };

    function replay() {
        // Replay journal on maps
        var m, xy, x, y;
        // all maps
        for (m in self.journal) {
            if (self.journal.hasOwnProperty(m)) {
                // all cells
                for (xy in self.journal[m]) {
                    if (self.journal[m].hasOwnProperty(xy)) {
                        x = xy.split(' ')[0];
                        y = xy.split(' ')[1];
                        GT.maps[m].ground[y][x] = self.journal[m][xy];
                    }
                }
            }
        }
    }
    replay();

    function log(aMap, aX, aY) {
        // Add map change to journal
        var map_name = typeof aMap === 'string' ? aMap : aMap.name,
            tiles = GT.maps[map_name].ground[aY][aX];
        if (!self.journal.hasOwnProperty(map_name)) {
            self.journal[map_name] = {};
        }
        self.journal[map_name][aX + ' ' + aY] = tiles;
    }

    self.change = function (aMap, aX, aY, aTiles, aFast) {
        // Change tiles on give position
        var map = typeof aMap === 'string' ? GT.maps[aMap] : aMap;
        if (typeof aTiles === 'string') {
            map.ground[aY][aX].push(aTiles);
        } else {
            map.ground[aY][aX] = aTiles;
        }
        log(aMap, aX, aY);
        if (map.name === GT.background.map) {
            GT.background.cell(map, aX, aY, self.debugDraw);
            if (!aFast) {
                GT.background.key = '';
                self.save();
            }
        }
    };

    self.changeArea = function (aMap, aX1, aY1, aX2, aY2, aTiles, aFast) {
        // Change rectangular area in map
        var x, y;
        for (x = aX1; x <= aX2; x++) {
            for (y = aY1; y <= aY2; y++) {
                self.change(aMap, x, y, aTiles, true);
            }
        }
        if (!aFast) {
            GT.background.key = '';
            self.save();
        }
    };

    self.checkSubmap = function (aCharacter) {
        // Check if character is near edge, if so move to neigbouring map
        var future_map,
            map = GT.maps[aCharacter.map];
        //console.log('checkSubmap', aCharacter, map, future_map);
        if (map.overlapTriggers) {
            // right
            if (aCharacter.x === map.overlapTriggers.right) {
                console.warn('OT right', GT.maps[aCharacter.map].neighbours.right);
                aCharacter.teleport(GT.maps[aCharacter.map].neighbours.right, 10, aCharacter.y, 'right');
            }
            // left
            if (aCharacter.x === map.overlapTriggers.left) {
                console.warn('OT left', GT.maps[aCharacter.map].neighbours.left);
                future_map = GT.maps[GT.maps[aCharacter.map].neighbours.left];
                aCharacter.teleport(GT.maps[aCharacter.map].neighbours.left, future_map.width - 10, aCharacter.y, 'left');
            }
            // up
            if (aCharacter.y === map.overlapTriggers.up) {
                console.warn('OT up', GT.maps[aCharacter.map].neighbours.up);
                future_map = GT.maps[GT.maps[aCharacter.map].neighbours.up];
                aCharacter.teleport(GT.maps[aCharacter.map].neighbours.up, aCharacter.x, future_map.height - 10, 'up');
            }
            // down
            if (aCharacter.y === map.overlapTriggers.down) {
                console.warn('OT down', GT.maps[aCharacter.map].neighbours.down);
                aCharacter.teleport(GT.maps[aCharacter.map].neighbours.down, aCharacter.x, 10, 'down');
            }
        }
    };

    self.walkableTileInCell = function (aMap, aX, aY) {
        // Slow function to test if cell is walkable
        var i, tiles = aMap.ground[aY][aX];
        for (i = 0; i < tiles.length; i++) {
            if (!GT.tiles[tiles[i]].walkable) {
                return false;
            }
        }
        return true;
    };

    return self;
}());

