// Mission functions and all missions
"use strict";
// globals: document, window, DH, setTimeout, Android, setInterval

var GT = GT || {};

GT.missions = GT.missions || {};

GT.mission = (function () {
    // Mission functions
    var self = {};
    self.major = DH.storage.readNumber('GT.mission.major', 0);
    self.missions = {};
    self.currentObjective = '';

    // set character defaults
    GT.characters.defaults.bat = { health: 120 };
    GT.characters.defaults.skeleton = { health: 500 };
    GT.characters.defaults.spider = { health: 150 };
    GT.characters.defaults.frog = { health: 100 };
    GT.characters.defaults.slime = { health: 300 };
    GT.characters.defaults.rat = { health: 50 };
    GT.characters.defaults.ghost = { health: 99999 };

    self.save = function () {
        // Save mission progress
        DH.storage.writeNumber('GT.mission.major', self.major);
    };

    self.reset = function (aMajor) {
        // Reset to another mission and reload
        self.major = typeof aMajor === 'number' ? aMajor : 0;
        self.save();
        document.location.reload();
    };

    self.next = function () {
        // Run next mission
        self.major++;
        self.save();
        console.log('Next major is ' + self.major);
        self.run();
    };

    self.run = function () {
        // Run current mission
        console.info('GT.mission.run', self.major);
        if (!GT.missions.hasOwnProperty(self.major)) {
            throw "Unknown mission " + self.major;
        }
        GT.missions[self.major]();
    };

    // mission objective checker
    self.checkObjective = undefined;
    setInterval(function () {
        if (self.checkObjective && !GT.pause) {
            self.checkObjective();
        }
    }, 1000);

    self.objective = function (aSubtitle, aTitle) {
        // Change mission objective label
        var o = document.getElementById('objective'),
            h1 = document.getElementById('objective_title'),
            h2 = document.getElementById('objective_subtitle');
        h1.textContent = aTitle || "Objective";
        h2.textContent = aSubtitle;
        self.currentObjective = aSubtitle;
        if (!aSubtitle) {
            o.style.visibility = 'hidden';
        }
    };

    self.placeOrNpc = function (aMap, aPlaceOrNpc) {
        // Return place, if there is no such place try npc by the same name
        var p = GT.places.byName[aMap + ' ' + aPlaceOrNpc];
        if (!p) {
            p = GT.maps[aMap].npc[aPlaceOrNpc];
        }
        return p;
    };

    self.destination = function (aMap, aPlace, aDistance, aCallback) {
        // Set objective to be given destination
        console.log('Setting destination objective', aMap, aPlace, aDistance);
        GT.mission.checkObjective = function () {
            var pp, d;
            // must be on the same map
            if (GT.characters.player.map !== aMap) {
                return;
            }
            pp = self.placeOrNpc(aMap, aPlace);
            d = GT.characters.player.distanceTo(pp.x, pp.y);
            //console.log('destination', aMap + ' ' + aPlace, 'd', d, 'limit', aDistance);
            if (d <= aDistance) {   // && GT.mission.enemiesAlive(GT.mission.enemies) <= 0
                self.checkObjective = undefined;
                self.objective('');
                if (aCallback) {
                    aCallback(aMap, aPlace, aDistance);
                }
            }
        };
    };

    self.randomEnemies = function (aMap, aX1, aY1, aX2, aY2, aBase, aAmount, aOnUpdateCallback) {
        // Create random enemies in given rectangle
        var e = [], i, x, y, j, c;
        for (i = 0; i < aAmount; i++) {
            // do not summon them on nowalk area
            for (j = 0; j < 20; j++) {
                x = Math.floor(aX1 + (aX2 - aX1) * Math.random());
                y = Math.floor(aY1 + (aY2 - aY1) * Math.random());
                if (GT.map.walkableTileInCell(GT.maps[aMap], x, y)) {
                    break;
                }
            }
            // enemy
            c = GT.character(GT.characters.uid(), aMap, x, y, aBase);
            c.acceptEvents = false;
            e.push(c);
            // callback for single enemy
            c.onUpdate = aOnUpdateCallback;
        }
        return e;
    };

    self.enemy = function (aMap, aX, aY, aBase, aOnUpdateCallback) {
        // Create single enemy
        self.randomEnemies(aMap, aX, aY, aX, aY, aBase, 1, aOnUpdateCallback);
    };

    self.enemiesAlive = function (aEnemies) {
        // Return number of alive enemies
        var i, a = 0;
        for (i = 0; i < aEnemies.length; i++) {
            if (aEnemies[i].health > 0) {
                a++;
            }
        }
        return a;
    };

    self.gameOver = function () {
        // Show game over and restart game
        GT.gallery('mission/gameover.png', '', '', function () {
            document.location.reload();
        }, true).style.imageRendering = 'pixelated';
        GT.metrics.missionEnd(false);
    };

    self.setForward = function (aMap, aPlace, aDx, aDy) {
        // Set forward button callback
        var pp = self.placeOrNpc(aMap, aPlace);
        console.log('Forward place', pp);
        GT.mission.forward = function () {
            GT.characters.player.teleport(aMap, pp.x + (aDx || 0), pp.y + (aDy || 0), 'down');
        };
        document.getElementById('fwd').onclick = GT.mission.forward;
    };

    self.controlsHide = function () {
        // Hide controls
        self.controls = false;
        GT.touchpad.hide();
        document.getElementById('objective').style.visibility = 'hidden';
        document.getElementById('attack').style.visibility = 'hidden';
        document.getElementById('minimap').style.visibility = 'hidden';
        document.getElementById('health').style.visibility = 'hidden';
        document.getElementById('herb').style.visibility = 'hidden';
    };

    self.controlsShow = function () {
        // Show controls
        self.controls = true;
        GT.touchpad.show();
        // dagger only visible in missions 2 to 11
        if (self.major >= 2 && self.major <= 11) {
            document.getElementById('attack').style.visibility = 'visible';
        }
        document.getElementById('minimap').style.visibility = 'visible';
        document.getElementById('objective').style.visibility = self.currentObjective ? 'visible' : 'hidden';
        document.getElementById('health').style.visibility = 'visible';
        document.getElementById('herb').style.visibility = GT.characters.player.herb > 0 ? 'visible' : 'hidden';
        document.getElementById('herbbutton').textContent = GT.characters.player.herb;
        self.sort();
    };

    self.playerSave = function () {
        // Save player's position before the end of the mission
        var o = {
            map: GT.characters.player.map,
            x: GT.characters.player.x,
            y: GT.characters.player.y,
            dir: GT.characters.player.dir,
            herb: GT.characters.player.herb
        };
        DH.storage.writeObject('GT.mission.player', o);
    };

    self.playerLoad = function (aMaxDistance) {
        // Restore player's position at the begining of the mission
        var o = DH.storage.readObject('GT.mission.player', {}),
            p = GT.characters.player;
        if (o.map === p.map && o.x && o.y && o.dir && (p.distanceTo(o.x, o.y) <= aMaxDistance)) {
            p.teleport(o.map, o.x, o.y, o.dir);
        }
        GT.characters.player.herb = o.herb || 0;
    };

    self.unlock = function () {
        // Mark current mission as unlocked
        DH.storage.writeBoolean('GT.mission.unlock.' + self.major, true);
    };

    self.begin = function (aMap, aPlace, aMinDistance) {
        // Standard mission begin
        self.controlsHide();
        // create player
        GT.characters.clear();
        self.enemies = [];
        var p = self.placeOrNpc(aMap, aPlace), a, b;
        GT.character('Player', aMap, p.x, p.y, 'boy').setPlayer();
        self.playerLoad(aMinDistance || 2);
        GT.background.load(aMap);
        GT.background.key = '';
        // make player collects herbs
        window.player = GT.characters.player;
        GT.characters.player.onWalk = function (c) {
            var he;
            if (GT.maps[c.map].ground[c.y][c.x].indexOf('herb-ground') >= 0) {
                GT.map.change(c.map, c.x, c.y, ['grass1']);
                if (GT.characters.player.herb <= 0) {
                    GT.characters.player.herb = 0;
                }
                GT.characters.player.herb++;
                // show number of herbs
                he = document.getElementById('herbbutton');
                he.innerText = GT.characters.player.herb;
                he.style.visibility = 'visible';
                GT.metrics.mission.herbsFound++;
            }
        };
        // make sure herbs are planted since mission 5 (both quarry and garden)
        if (self.major >= 5) {
            // garden
            a = GT.mission.placeOrNpc('ghosttown', 'garden_origin');
            b = GT.mission.placeOrNpc('ghosttown', 'garden_end');
            GT.map.changeArea('ghosttown', a.x, a.y, b.x, b.y, ['grass1', 'herb-ground']);
        }
        // quarry
        a = GT.mission.placeOrNpc('ghosttown', 'herb');
        GT.map.change('ghosttown', a.x - 1, a.y, ['grass1', 'herb-ground']);
        GT.map.change('ghosttown', a.x, a.y, ['grass1', 'herb-ground']);
        GT.map.change('ghosttown', a.x + 1, a.y, ['grass1', 'herb-ground']);
        GT.map.change('ghosttown', a.x, a.y + 1, ['grass1', 'herb-ground']);
        // metrics
        GT.metrics.missionBegin(self.major);
    };

    self.end = function (aCallback) {
        // Save players position, do something or move to next mission
        self.controlsHide();
        self.playerSave();
        self.unlock(self.major);
        if (typeof aCallback === 'function') {
            aCallback();
        } else {
            setTimeout(self.next, 500);
        }
        GT.metrics.missionEnd(true);
    };

    self.npc = function (aName, aMap, aPlace, aBase, aDir, aCallback) {
        // Create one npc
        var p = self.placeOrNpc(aMap, aPlace),
            ch = GT.character(aName, aMap, p.x, p.y, aBase);
        ch.turn(aDir || 'down');
        if (aCallback) {
            aCallback(ch);
        }
        return ch;
    };

    self.monster = function (aMap, aPlace) {
        // Create one monster from map npc
        var p = self.placeOrNpc(aMap, aPlace),
            b = aPlace.match(/[a-z]+/)[0],
            ch = GT.character(aPlace, aMap, p.x, p.y, b);
        ch.turn('down');
        ch.onUpdate = GT.enemy.onUpdate;
        return ch;
    };

    self.monsters = function (aMap, aBase, aCount) {
        // Create multiple monsters from map npc
        var i;
        for (i = 1; i < aCount; i++) {
            self.monster(aMap, aBase + i);
        }
    };

    self.walk = function (aName, aPath, aCallback, aCallback2) {
        // Walk NPC with given name
        GT.characters.names[aName].walkPath(aPath, aCallback, aCallback2);
    };

    self.chestOpen = function (aMap, aPlace, aCallback) {
        // Animate opening treasure chest
        var p = self.placeOrNpc(aMap, aPlace),
            tiles = GT.maps.library.ground[p.y][p.x],
            i = tiles.indexOf('chest1');
        // remove chest
        if (i > 0) {
            tiles.splice(i, 1);
        }
        // opening
        setTimeout(function () {
            var tiles2 = tiles.slice();
            tiles2.push('chest2');
            GT.map.change('library', p.x, p.y, tiles2);
        }, 400);
        setTimeout(function () {
            var tiles2 = tiles.slice();
            tiles2.push('chest3');
            GT.map.change('library', p.x, p.y, tiles2);
        }, 800);
        setTimeout(function () {
            var tiles2 = tiles.slice();
            tiles2.push('chest4');
            GT.map.change('library', p.x, p.y, tiles2);
        }, 1200);
        // callback after open
        if (typeof aCallback === 'function') {
            setTimeout(aCallback, 1600);
        }
    };

    self.sort = function (aDebug) {
        // Sort optimally characters by their order
        var order = {
            // down
            frog: 1,
            crystals: 1,
            slime: 1,
            spider: 1,
            rat: 1,
            // middle
            girl: 2,
            skeleton: 2,
            // player
            boy: 3,
            // up
            bat: 4,
            ghost: 4
        };
        GT.characters.optimalRenderingOrder(order, aDebug);
    };

    self.defaultEnemies = function () {
        // Place default enemies on the map
        // slimes on square
        GT.mission.randomEnemies('ghosttown', 42, 47, 62, 57, 'slime', 10, GT.enemy.onUpdate);
        // slimes in bank
        GT.mission.enemy('ghosttown', 57, 53, 'slime', GT.enemy.onUpdate);
        GT.mission.enemy('ghosttown', 58, 53, 'slime', GT.enemy.onUpdate);
        GT.mission.enemy('ghosttown', 59, 53, 'slime', GT.enemy.onUpdate);
        // rats near fisherman
        GT.mission.randomEnemies('ghosttown', 64, 47, 68, 67, 'rat', 5, GT.enemy.onUpdate);
        // few frogs near jane's pond and near lake
        GT.mission.enemy('ghosttown', 43, 65, 'frog', GT.enemy.onUpdate);
        GT.mission.enemy('ghosttown', 46, 65, 'frog', GT.enemy.onUpdate);
        GT.mission.enemy('ghosttown', 58, 68, 'frog', GT.enemy.onUpdate);
        GT.mission.enemy('ghosttown', 60, 64, 'frog', GT.enemy.onUpdate);
        // one bat in front of quarry
        GT.mission.enemy('ghosttown', 50, 72, 'bat', GT.enemy.onUpdate);
        // bats in quarry
        GT.mission.enemy('quarry', 4, 4, 'bat', GT.enemy.onUpdate);
        GT.mission.enemy('quarry', 5, 3, 'bat', GT.enemy.onUpdate);
        GT.mission.enemy('quarry', 7, 3, 'bat', GT.enemy.onUpdate);
        GT.mission.enemy('quarry', 8, 2, 'bat', GT.enemy.onUpdate);
        GT.mission.enemy('quarry', 5, 4, 'bat', GT.enemy.onUpdate);
        GT.mission.enemy('quarry', 5, 5, 'bat', GT.enemy.onUpdate);
        GT.mission.enemy('quarry', 5, 6, 'bat', GT.enemy.onUpdate);
        GT.mission.enemy('quarry', 12, 5, 'bat', GT.enemy.onUpdate);
        // crystals in ken basement (so that when player goes there he knows they are there and they can be destroyed)
        if (self.major < 9) {
            GT.mission.npc('Crystals1', 'quarry', 'crystals1', 'crystals');
            GT.mission.npc('Crystals2', 'basement_ken', 'crystals2', 'crystals');
            GT.mission.npc('Crystals3', 'basement_ken', 'crystals3', 'crystals');
        }
        // one bat below quarry
        GT.mission.enemy('ghosttown', 53, 84, 'bat', GT.enemy.onUpdate);
        // spiders in ken's basement
        self.monsters('basement_ken', 'spider', 12);
        // skeletons                                                     // 15
        GT.mission.randomEnemies('ghosttown', 5, 52, 17, 60, 'skeleton', 35, GT.enemy.onUpdate);
        GT.mission.sort();
    };

    self.lightning = function (aCallback) {
        // Flash background 2-3 times
        var e = document.getElementById('background_canvas');
        e.style.transition = 'opacity 0.1s linear';
        setTimeout(function () {
            e.style.opacity = 0;
        }, 1);
        setTimeout(function () {
            e.style.opacity = 1;
        }, 100);
        setTimeout(function () {
            e.style.opacity = 0;
        }, 200);
        setTimeout(function () {
            e.style.opacity = 1;
        }, 300);
        setTimeout(function () {
            e.style.opacity = 0;
        }, 400);
        setTimeout(function () {
            e.style.opacity = 1;
        }, 500);
        setTimeout(function () {
            if (aCallback) {
                aCallback();
            }
        }, 600);
    };

    self.minimap = function (aCallback) {
        // Show current mission minimap
        GT.render();
        GT.galleryMap('mission/map' + self.major + '.png', self.currentObjective, typeof aCallback === 'function' ? aCallback : null);
        GT.metrics.mission.minimap++;
    };

    return self;
}());

// All missions

GT.missions["0"] = function () {
    // Intro
    GT.mission.begin('ghosttown', 'under_throne_left');
    GT.mission.objective('Introduction');
    GT.mission.setForward('ghosttown', 'fisherman', -3, 0);
    GT.mission.npc('Fisherman', 'ghosttown', 'fisherman', 'boy', 'down');
    GT.mission.npc('Mayor', 'ghosttown', 'throne', 'girl', 'down');
    GT.mission.walk('Mayor', 'ppppdDrRdDl', function () {
        GT.bubbles('Mayor',
            ["Hello, I am the Mayor of this town and I need your help",
                "Magic book from town library got lost",
                "Somebody stole it and used it for evil purposes",
                "Dangerous creatures started appearing all over the town",
                "You must stop them! But first you need some weapons",
                "Fisherman Fred will give you dagger, I'll show you where he lives on this map"
                ],
            function () {
                // Walk Mayor back to throne
                GT.mission.walk('Mayor', 'uUlLuUd', GT.mission.end);
            });
    });
};

GT.missions["1"] = function () {
    // Getting to fisherman
    GT.mission.begin('ghosttown', 'under_throne_left');
    GT.mission.objective('Visit fisherman');
    GT.mission.setForward('ghosttown', 'fisherman', -3, 0);
    GT.mission.npc('Fisherman', 'ghosttown', 'fisherman', 'boy', 'down');
    GT.mission.npc('Mayor', 'ghosttown', 'throne', 'girl', 'down');
    // players have trouble getting past first level
    GT.characters.player.health = 130;
    // Show map
    GT.mission.minimap(function () {
        GT.bubbles('Mayor', ['Run as fast as you can!'], function () {
            // show controls
            GT.mission.controlsShow();
            // show enemies
            GT.mission.defaultEnemies();
            // simply reaching fisherman will end mission
            GT.mission.destination('ghosttown', 'fisherman', 2, function () {
                GT.mission.controlsHide();
                GT.bubbles('Fisherman',
                    ["You made it! I have a dagger for you",
                        "It's not fancy but better than nothing",
                        "With dagger you can kill those evil creatures",
                        "By the way, my niece Jane lives in town, she needs some food, can you bring her these fish",
                        "I show you on the map where Jane lives"
                        ],
                    GT.mission.end);
            });

        });
    });
};

GT.missions["2"] = function () {
    // Deliver fish to Jane
    GT.mission.begin('ghosttown', 'fisherman_room');
    GT.mission.objective('Deliver fish to Jane');
    GT.mission.setForward('ghosttown', 'jane', 0, 3);
    GT.mission.npc('Fisherman', 'ghosttown', 'fisherman', 'boy', 'down');
    GT.mission.npc('Jane', 'ghosttown', 'jane_bed', 'girl', 'down');
    // show map
    GT.mission.minimap(function () {
        // just for fun put fisherman to sleep
        GT.bubbles('Fisherman', ["Please go deliver the fish, I'm gonna take a short nap"], function (c) {
            c.turn('sleep');
            c.ry -= 0.3;
            // show controls
            GT.mission.controlsShow();
            // show enemies
            GT.mission.defaultEnemies();
            // simply reaching jane will end mission
            GT.mission.destination('ghosttown', 'jane_bed', 2, function () {
                GT.mission.controlsHide();
                GT.bubbles('Jane',
                    ["Thank you for all the fish",
                        "It must been tough getting through all the monsters",
                        "If you'll ever get injured, you can heal yourself with medicinal herb",
                        "It grows on the quarry mountain. If you bring me some I can plant it in my garden.",
                        "Then you can take it from my garden any time you walk by",
                        "I'll show you on the map"],
                    GT.mission.end);
            });
        });
    });
};

GT.missions["3"] = function () {
    // Find herb
    GT.mission.begin('ghosttown', 'jane_room');
    GT.mission.objective('Find herb');
    GT.mission.setForward('ghosttown', 'herb', -3, 0);
    GT.mission.npc('Jane', 'ghosttown', 'jane_bed', 'girl', 'down');
    // clear garden
    var p = GT.places.byName["ghosttown garden_origin"];
    GT.map.changeArea('ghosttown', p.x, p.y, p.x + 3, p.y + 1, ['grass4']);
    // map
    GT.mission.minimap(function () {
        // good luck
        GT.bubbles('Jane', ['Good luck and be careful!'], function () {
            // show controls
            GT.mission.controlsShow();
            // show enemies
            GT.mission.defaultEnemies();
            // simply reaching herb will end mission
            GT.mission.destination('ghosttown', 'herb', 1, function () {
                // hide controls
                GT.mission.controlsHide();
                GT.bubbles('Player', ["Ok, these herbs are perfect", "Now back to Jane's garden"], GT.mission.end);
            });
        });
    });
};

GT.missions["4"] = function () {
    // Bring herb back
    GT.mission.begin('ghosttown', 'herb');
    GT.mission.objective('Bring herb back');
    GT.mission.setForward('ghosttown', 'garden', 2, 2);
    GT.mission.npc('Jane', 'ghosttown', 'garden', 'girl', 'down');
    // make sure garden is empty
    var p = GT.places.byName["ghosttown garden_origin"];
    GT.map.changeArea('ghosttown', p.x, p.y, p.x + 3, p.y + 1, ['grass4']);
    // map
    GT.mission.minimap(function () {
        // show controls
        GT.mission.controlsShow();
        // show enemies
        GT.mission.defaultEnemies();
        // simply reaching garden will end mission
        GT.mission.destination('ghosttown', 'garden', 2, function () {
            // hide controls
            GT.mission.controlsHide();
            // move player under garden for the dialog
            p = GT.mission.placeOrNpc('ghosttown', 'garden');
            GT.characters.player.goto(p.x, p.y + 1, 'up');
            GT.bubbles('Jane',
                ["Thank you for the herbs",
                    "I will plant them in my garden",
                    "Use the herbs when you are injured, they will heal you instantly"],
                function () {
                    GT.mission.walk('Jane', 'uUlLuU2prRu2prRu2prRu2prpdpDu2plLu2plLu2plLu2prRdD',
                        function () {
                            GT.bubbles('Jane',
                                ["Now go back to fisherman, he needs your help", "Here is the map"], GT.mission.end);
                        },
                        function (aCharacter) {
                            // plant herb
                            GT.map.change('ghosttown', aCharacter.x, aCharacter.y, ['grass1', 'herb-ground']);
                        });
                });
        });
    });
};

GT.missions["5"] = function () {
    // Visit fisherman
    GT.mission.begin('ghosttown', 'jane_garden_under');
    GT.mission.objective('Visit fisherman');
    GT.mission.setForward('ghosttown', 'fisherman', -3, 0);
    GT.mission.npc('Jane', 'ghosttown', 'garden', 'girl', 'down');
    GT.mission.npc('Fisherman', 'ghosttown', 'fisherman_bed', 'boy', 'down');
    // make sure herbs are planted since mission 5
    var rs = GT.places.byName["ghosttown river_source"];
    // make river dry
    GT.river(false);
    // river dam
    GT.map.change('ghosttown', rs.x, rs.y, ['mud', 'dam']);
    GT.mission.minimap(function () {
        // show controls
        GT.mission.controlsShow();
        // show enemies
        GT.mission.defaultEnemies();
        // simply reaching garden will end mission
        GT.mission.destination('ghosttown', 'fisherman', 2, function () {
            // hide controls
            GT.mission.controlsHide();
            // speech
            GT.bubbles('Fisherman',
                ["I need your help, the river dried up",
                    "Evil frogs build a dam and stopped the river",
                    "You must kill those frogs",
                    "Here is the map"], GT.mission.end);
        });
    });
};

GT.missions["6"] = function () {
    // Fix dry river
    GT.mission.begin('ghosttown', 'fisherman_room');
    GT.mission.objective('Fix dry river');
    GT.mission.setForward('ghosttown', 'river_source', -2, -2);
    GT.mission.npc('Fisherman', 'ghosttown', 'fisherman_bed', 'boy', 'down');
    // make sure it's dry
    GT.river(false, null, true);
    // explain mission
    GT.mission.minimap(function () {
        // show enemies
        GT.mission.defaultEnemies();
        GT.mission.enemies.push(GT.mission.npc('frog1', 'ghosttown', 'frog1', 'frog', 'down'));
        GT.mission.enemies.push(GT.mission.npc('frog2', 'ghosttown', 'frog2', 'frog', 'down'));
        GT.mission.enemies.push(GT.mission.npc('frog3', 'ghosttown', 'frog3', 'frog', 'down'));
        GT.mission.enemies.push(GT.mission.npc('frog4', 'ghosttown', 'frog4', 'frog', 'down'));
        GT.mission.enemies.push(GT.mission.npc('frog5', 'ghosttown', 'frog5', 'frog', 'down'));
        GT.mission.enemies.push(GT.mission.npc('frog6', 'ghosttown', 'frog6', 'frog', 'down'));
        GT.mission.enemies.push(GT.mission.npc('frog7', 'ghosttown', 'frog7', 'frog', 'down'));
        GT.mission.enemies.push(GT.mission.npc('frog8', 'ghosttown', 'frog8', 'frog', 'down'));
        GT.mission.enemies.push(GT.mission.npc('frog9', 'ghosttown', 'frog9', 'frog', 'down'));
        GT.mission.enemies.push(GT.mission.npc('frog10', 'ghosttown', 'frog10', 'frog', 'down'));
        // make those frogs attack as well
        var i;
        for (i = 0; i < GT.mission.enemies.length; i++) {
            GT.mission.enemies[i].onUpdate = GT.enemy.onUpdate;
        }
        // main frog will be stationary
        GT.mission.frog = GT.mission.npc('Frog', 'ghosttown', 'river_source', 'frog');
        GT.mission.enemies.push(GT.mission.frog);
        // show controls
        GT.mission.controlsShow();
        // check objective
        GT.mission.checkObjective = function () {
            // did player killed all frogs?
            if (GT.mission.enemiesAlive(GT.mission.enemies) <= 0) {
                // disable check
                GT.mission.checkObjective = undefined;
                // hide controls
                GT.mission.controlsHide();
                // remove dead
                GT.characters.removeDead();
                GT.mission.enemies = [];
                // show main frog dead
                GT.mission.frog = GT.mission.npc('Frog', 'ghosttown', 'river_source', 'frog');
                GT.mission.frog.turn('dead');
                GT.mission.frog.health = 0;
                // speech
                GT.bubbles('Frog',
                    ['You got me!',
                        'All I ever wanted was rule the world with the Ghost',
                        'He stole the magic book and summoned all monsters',
                        'Oops. I shouldn\'t have said that. Forget about it!',
                        'Aaargh...'
                        ],
                    function () {
                        // remove frog
                        GT.mission.frog.base('invisible');
                        // restore river and next mission
                        GT.river(true, GT.mission.end);
                    });
            }
        };
    });
};

GT.missions["7"] = function () {
    // Tell mayor about Ghost
    GT.mission.begin('ghosttown', 'river_source_player', 5);
    GT.mission.objective('Tell mayor about Ghost');
    GT.mission.setForward('ghosttown', 'throne', 0, 3);
    GT.mission.npc('Mayor', 'ghosttown', 'throne', 'girl', 'down');
    // explain mission
    GT.bubbles('Player',
        ["I must tell mayor it was Ghost who stole the magic book!"],
        function () {
            GT.mission.minimap(function () {
                // show controls
                GT.mission.controlsShow();
                // show enemies
                GT.mission.defaultEnemies();
                // reaching destination will end mission
                GT.mission.destination('ghosttown', 'throne', 2, function () {
                    // hide controls
                    GT.mission.controlsHide();
                    // speech
                    GT.bubbles('Player', [
                        'Frog told me who stole the magic book when I defeated it',
                        'It was Ghost! Should I kill him?'],
                        function () {
                            GT.bubbles('Mayor',
                                ['No! Dagger is not enough to kill Ghost.',
                                    'You need a special weapon, a magic wand!',
                                    'You need three crystals to make magic wand, go to the land office',
                                    'Surveyor from land office knows where the crystals are',
                                    'Here is the map'], GT.mission.end);
                        });
                });
            });
        });
};

GT.missions["8"] = function () {
    // Visit surveyor
    GT.mission.begin('ghosttown', 'under_throne_left');
    GT.mission.objective('Visit surveyor');
    GT.mission.setForward('ghosttown', 'surveyor', 0, 3);
    GT.mission.npc('Mayor', 'ghosttown', 'throne', 'girl', 'down');
    GT.mission.npc('Surveyor', 'ghosttown', 'surveyor', 'girl', 'down');
    // show map
    GT.mission.minimap(function () {
        // good luck
        GT.bubbles('Mayor', ['Good luck!'], function () {
            // show controls
            GT.mission.controlsShow();
            // show enemies
            GT.mission.defaultEnemies();
            // reaching destination will end mission
            GT.mission.destination('ghosttown', 'surveyor', 2, function () {
                // hide controls
                GT.mission.controlsHide();
                // chat
                GT.bubbles('Player', ['Hello, I need 3 crystals to make a magic wand'], function () {
                    GT.bubbles('Surveyor', ['Crystals only grow underground',
                        'One crystal is in the quarry, two crystals are in Ken\'s basement',
                        'Be very careful, there are many bats and spiders underground!',
                        'You can use dagger to free crystals from the ground',
                        'When you have 3 crystals give them to mayor, she will make the wand for you',
                        'Here is the map'], GT.mission.end);
                });
            });
        });
    });
};

GT.missions["9"] = function () {
    // Find 3 crystals
    GT.mission.begin('ghosttown', 'surveyor_desk');
    GT.mission.objective('Find 3 crystals');
    GT.mission.setForward('quarry', 'crystals1', -2, -2);
    GT.mission.npc('Surveyor', 'ghosttown', 'surveyor', 'girl', 'down');
    GT.mission.npc('Mayor', 'ghosttown', 'throne', 'girl', 'down');
    // turn player facing up
    GT.characters.player.turn('up');
    // place crystals in quarry and ken's basement
    GT.mission.crystals1 = GT.mission.npc('Crystals1', 'quarry', 'crystals1', 'crystals');
    GT.mission.crystals2 = GT.mission.npc('Crystals2', 'basement_ken', 'crystals2', 'crystals');
    GT.mission.crystals3 = GT.mission.npc('Crystals3', 'basement_ken', 'crystals3', 'crystals');
    GT.mission.remaining = 3;

    function onDie(aCharacter) {
        // event when crystal is destroyed
        console.log('onDie', aCharacter.name);
        GT.mission.remaining--;
        // say remaining
        if (GT.mission.remaining > 0) {
            GT.mission.controlsHide();
            GT.bubbles('Player', ['I need ' + GT.mission.remaining + ' more'], GT.mission.controlsShow);
            if (GT.mission.remaining === 2) {
                GT.mission.setForward('basement_ken', 'crystals2', 2, 0);
            }
        }
    }

    GT.mission.crystals1.onDie = onDie;
    GT.mission.crystals2.onDie = onDie;
    GT.mission.crystals3.onDie = onDie;
    // objective checker function
    GT.mission.checkObjective = function () {
        // all 3 collected?
        if (GT.mission.remaining === 0) {
            GT.mission.checkObjective = undefined;
            GT.mission.controlsHide();
            GT.bubbles('Player', ['Ok, back to mayor'], function () {
                GT.mission.controlsShow();
                GT.mission.objective('Back to mayor');
                GT.mission.setForward('ghosttown', 'throne', 0, 3);
                // must reach mayor
                GT.mission.destination('ghosttown', 'throne', 2, function () {
                    // speech
                    GT.mission.controlsHide();
                    GT.bubbles('Mayor', ["Thank you for the crystals", "I will make the magic wand now..."], function () {
                        // flash screen
                        GT.mission.lightning(function () {
                            // make the magic wand
                            GT.effects.magic(GT.characters.names.Mayor, function () {
                                // speech
                                GT.bubbles('Mayor',
                                    ['Success! Here is your magic wand!',
                                        'It is only effective against Ghost',
                                        'Ghost lives in the cave on the cemetery',
                                        'Cemetery is full of skeletons, be very careful',
                                        'Here is the map'],
                                    function () {
                                        GT.mission.end();
                                    });
                            });
                        });
                    });
                });
            });
        }
    };
    // show map
    GT.mission.minimap(function () {
        // show controls
        GT.mission.controlsShow();
        // show enemies
        GT.mission.defaultEnemies();
    });
};

GT.missions["10"] = function () {
    // Go to cemetery
    GT.mission.begin('ghosttown', 'under_throne_left');
    GT.mission.objective('Go to cemetery');
    GT.mission.setForward('ghosttown', 'tomb', 0, 3);
    GT.mission.npc('Mayor', 'ghosttown', 'throne', 'girl', 'down');
    GT.mission.npc('Ghost', 'tomb', 'ghost', 'ghost', 'down');
    // show map
    GT.mission.minimap(function () {
        // good luck
        GT.bubbles('Mayor', ['Good luck!'], function () {
            // show controls
            GT.mission.controlsShow();
            // show enemies
            GT.mission.defaultEnemies();
            // reaching destination will end mission
            GT.mission.destination('tomb', 'doors', 2, GT.mission.end);
        });
    });
};

GT.missions["11"] = function () {
    // Kill Ghost
    GT.mission.begin('tomb', 'doors');
    GT.mission.objective('Kill Ghost');
    GT.mission.setForward('tomb', 'doors', 0, 0);
    GT.mission.npc('Mayor', 'ghosttown', 'throne', 'girl', 'down');
    var g = GT.mission.npc('Ghost', 'tomb', 'ghost', 'ghost', 'down');
    GT.characters.player.turn('up');
    // speech
    GT.bubbles('Player', ['I came here to kill you evil Ghost!'], function () {
        GT.bubbles('Ghost', ['Ha ha ha, you cannot kill me with a dagger!'], function () {
            GT.bubbles('Player', ['I have something better!'], function () {
                document.getElementById('attackbutton').style.backgroundImage = 'url(icon/magicwand32.png)';
                GT.bubbles('Player', ['Magic wand!'], function () {
                    GT.bubbles('Ghost', ['Oh oh!'], function () {
                        // show controls
                        GT.mission.controlsShow();
                        // ghost start attacking
                        g.onUpdate = GT.enemy.onUpdate;
                        // check objective
                        GT.mission.checkObjective = function () {
                            // was ghost killed?
                            if (GT.characters.names.Ghost.health <= 0) {
                                GT.mission.checkObjective = undefined;
                                GT.mission.controlsHide();
                                // speech
                                GT.bubbles('Ghost', ['You got me!', 'Now I can finally rest in peace'], function () {
                                    // turn ghost to skeleton
                                    GT.characters.names.Ghost.base('skeleton');
                                    GT.characters.names.Ghost.health = 100;
                                    GT.characters.names.Ghost.turn('down');
                                    // move to tomb stone
                                    var p = GT.mission.placeOrNpc('tomb', 'stone');
                                    GT.characters.names.Ghost.goto(p.x, p.y, 'down');
                                    // turn to sleeping boy on arrival
                                    GT.characters.names.Ghost.onStop = function () {
                                        GT.characters.names.Ghost.base('boy');
                                        GT.characters.names.Ghost.turn('sleep');
                                        // final words
                                        GT.bubbles('Ghost', ['My last wish is...', 'Please deliver magic book back to Mayor', 'And make sure nobody else can get to it'], GT.mission.end);
                                    };
                                });
                            }
                        };
                    });
                });
            });
        });
    });
};

GT.missions["12"] = function () {
    // Deliver book back to mayor
    GT.mission.begin('tomb', 'middle');
    GT.mission.objective('Return book to Mayor');
    GT.mission.setForward('ghosttown', 'throne', 0, 3);
    GT.mission.npc('Mayor', 'ghosttown', 'throne', 'girl', 'down');
    GT.mission.npc('Ghost', 'tomb', 'stone', 'boy', 'sleep');
    // make sure table is empty
    var p = GT.mission.placeOrNpc('library', 'table');
    GT.map.change('library', p.x, p.y, ['floor4', 'table']);
    // make sure chest is closed
    p = GT.mission.placeOrNpc('library', 'chest');
    GT.map.change('library', p.x, p.y, ['floor4', 'chest1']);
    // explain mission
    GT.bubbles('Player',
        ['This book is too dangerous',
            'I must return it to Mayor'], function () {
            // show map
            GT.mission.minimap(function () {
                // show controls
                GT.mission.controlsShow();
                // no enemies on last mission
                // reaching throne
                GT.mission.destination('ghosttown', 'throne', 2, function () {
                    GT.mission.controlsHide();
                    GT.mission.objective('');
                    // mayor speech
                    GT.bubbles('Mayor',
                        ['You are my hero! You saved our town once again', 'Follow me'],
                        function () {
                            GT.mission.controlsShow();
                            GT.characters.names.Mayor.walkPath('lLLpppppppprRRuUpppppppUUrRuUUlLd', function () {
                                console.log('Mayor sitting in library');
                            });
                        });
                    // reaching library
                    GT.mission.destination('library', 'table', 2, function () {
                        GT.mission.controlsHide();
                        GT.bubbles('Mayor', ['The book shall remain here', 'I will lock the door, nobody will enter this library'],
                            function () {
                                // put book on the table
                                var t = GT.mission.placeOrNpc('library', 'table');
                                GT.map.change('library', t.x, t.y, ['floor4', 'table', 'booksmall']);
                                // make sure mayor is on chair because her next walk is relative
                                GT.characters.names.Mayor.teleport('library', t.x, t.y - 1, 'down');
                                // move to chest
                                GT.mission.walk('Mayor', 'lLL', function () {
                                    // open chest
                                    GT.mission.chestOpen('library', 'chest', function () {
                                        // move down
                                        GT.mission.walk('Mayor', 'pppprRdDD', function () {
                                            // speak
                                            GT.bubbles('Mayor', ['I have something for you, follow me!'], function () {
                                                // move to hall
                                                GT.mission.controlsShow();
                                                GT.mission.walk('Mayor', 'DrRdDppppdDlLLpppprRRd', function () {
                                                    console.log('m in hall');
                                                });
                                            });
                                            // reaching hall again
                                            GT.mission.destination('ghosttown', 'throne', 2, function () {
                                                GT.mission.controlsHide();
                                                // walk player in front of throne
                                                var u = GT.mission.placeOrNpc('ghosttown', 'throne');
                                                GT.characters.player.goto(u.x, u.y + 2, 'up');
                                                // final speech
                                                GT.bubbles('Mayor', ['For the bravery and courage you showed us', 'and for saving our town once again', 'I am giving you Sword of Ghost town founder'], function () {
                                                    GT.mission.controlsShow();
                                                    GT.galleryItem('Ghost sword', 'mission/sword300.png', function () {
                                                        GT.bubbles('Mayor', ['You will need it in Ghost town 3: destruction of magic book', 'You can click on menu on the top if you want to replay the game', 'Thank you for playing this game. Bye bye.'], function () {
                                                            GT.mission.objective('');
                                                            GT.mission.end();
                                                        });
                                                    });
                                                });
                                            });
                                        });
                                    });
                                });
                            });
                    });
                });
            });
        });
};

GT.missions["13"] = function () {
    // ending
    GT.mission.begin('ghosttown', 'under_throne_left');
    GT.mission.objective('Game over', 'Menu');
    GT.mission.setForward('ghosttown', 'throne', 0, 3);
    GT.mission.npc('Mayor', 'ghosttown', 'throne', 'girl', 'down');
    GT.mission.npc('Fisherman', 'ghosttown', 'fisherman', 'boy', 'down');
    GT.mission.npc('Surveyor', 'ghosttown', 'surveyor', 'girl', 'down');
    GT.mission.npc('Jane', 'ghosttown', 'jane_bed', 'girl', 'down');
    GT.mission.controlsShow();
    GT.menu();
};

GT.missions["14"] = function () {
    // stopper
    console.warn('14 - no more missions');
};

GT.missions["20"] = function () {
    // testing mission
    GT.mission.begin('quarry', 'crystals1');
    GT.mission.objective('Testing mission');
    GT.mission.setForward('ghosttown', 'tomb', 0, 3);
    GT.mission.defaultEnemies();
    GT.mission.controlsShow();
    //GT.mission.npc('Mayor', 'ghosttown', 'throne', 'girl', 'down');
    //GT.mission.npc('Ghost', 'tomb', 'stone', 'boy', 'sleep');
};


